@@ -1,4 +1,4 @@
-/*	$OpenBSD: main.c,v 1.99 2023/02/08 17:22:10 kn Exp $	*/
+/*	$OpenBSD: main.c,v 1.100 2023/07/23 23:42:03 kn Exp $	*/
 
 /*
  * startup, main loop, environments and error handling
@@ -88,7 +88,11 @@ static const char *initcoms [] = {
 	"typeset", "-x", "SHELL", "PATH", "HOME", "PWD", "OLDPWD", NULL,
 	"typeset", "-ir", "PPID", NULL,
 	"typeset", "-i", "OPTIND=1", NULL,
+#ifndef SMALL
 	"eval", "typeset -i RANDOM MAILCHECK=\"${MAILCHECK-600}\" SECONDS=\"${SECONDS-0}\" TMOUT=\"${TMOUT-0}\"", NULL,
+#else
+	"eval", "typeset -i RANDOM SECONDS=\"${SECONDS-0}\" TMOUT=\"${TMOUT-0}\"", NULL,
+#endif /* SMALL */
 	"alias",
 	 /* Standard ksh aliases */
 	  "hash=alias -t",	/* not "alias -t --": hash -r needs to work */
@@ -308,7 +312,7 @@ main(int argc, char *argv[])
 #ifndef SMALL
 	/* setstr can't fail here */
 	setstr(global(version_param), ksh_version, KSH_RETURN_ERROR);
-	setstr(global("OKSH_VERSION"), "oksh 7.3", KSH_RETURN_ERROR);
+	setstr(global("OKSH_VERSION"), "oksh 7.4", KSH_RETURN_ERROR);
 #endif /* SMALL */
 
 	/* execute initialization statements */
@@ -619,7 +623,9 @@ shell(Source *volatile s, volatile int toplevel)
 		if (interactive) {
 			got_sigwinch = 1;
 			j_notify();
+#ifndef SMALL
 			mcheck();
+#endif /* SMALL */
 			set_prompt(PS1);
 		}
 
